'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.notifyErrorListeners = exports.notifyBusyListeners = exports.notifySuccessListeners = exports.addErrorListener = exports.addBusyListener = exports.addSuccessListener = undefined;

var _createAction = require('./createAction');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var globalSuccessListeners = [];
var globalErrorListeners = [];
var globalBusyListeners = [];
var individualListeners = {};

function addListener(globalListeners, typifier) {
  switch (arguments.length <= 2 ? 0 : arguments.length - 2) {
    // global listener
    case 1:
      globalListeners.push(arguments.length <= 2 ? undefined : arguments[2]);
      break;

    // action listener
    case 2:
      var listener = arguments.length <= 3 ? undefined : arguments[3];
      var actionType = typifier(arguments.length <= 2 ? undefined : arguments[2]);

      if (!individualListeners.hasOwnProperty(actionType)) individualListeners[actionType] = [listener];else individualListeners[actionType].push(listener);

      break;

    default:
      // Invalid arguments
      throw new Error('Invalid number of arguments');
  }
}

function notifyListeners(globalListeners, typifier, action, payload, args) {
  var actionType = typifier(action);

  // Notify global listeners
  globalListeners.forEach(function (fn) {
    return fn.apply(null, [action, payload].concat(_toConsumableArray(args)));
  });

  // Notify individual listeners
  if (individualListeners.hasOwnProperty(actionType)) {
    individualListeners[actionType].forEach(function (fn) {
      return fn.apply(null, [payload].concat(_toConsumableArray(args)));
    });
  }
}

var addSuccessListener = exports.addSuccessListener = addListener.bind(null, globalSuccessListeners, _createAction.successType);
var addBusyListener = exports.addBusyListener = addListener.bind(null, globalBusyListeners, _createAction.busyType);
var addErrorListener = exports.addErrorListener = addListener.bind(null, globalErrorListeners, _createAction.errorType);

var notifySuccessListeners = exports.notifySuccessListeners = notifyListeners.bind(null, globalSuccessListeners, _createAction.successType);
var notifyBusyListeners = exports.notifyBusyListeners = notifyListeners.bind(null, globalBusyListeners, _createAction.successType);
var notifyErrorListeners = exports.notifyErrorListeners = notifyListeners.bind(null, globalErrorListeners, _createAction.successType);